/*
 *  thermal.h - ACPI Thermal Zone Driver ($Revision: 1 $)
 *
 *  Copyright (C) 2001, 2002 Andy Grover <andrew.grover@intel.com>
 *  Copyright (C) 2001, 2002 Paul Diefenbaugh <paul.s.diefenbaugh@intel.com>
 *  Copyright (C) 2001, 2002 Sujith Thomas <sujith.thomas@intel.com>

 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation;  version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This driver fully implements the ACPI thermal policy as described in the
 *  ACPI 2.0 Specification.
 *
 *  TBD: 1. Implement passive cooling hysteresis.
 *       2. Enhance passive cooling (CPU) states/limit interface to support
 *          concepts of 'multiple limiters', upper/lower limits, etc.
 *  Modified: 1. Provide infrastructure from kernel to enable user-space
 *              thermal management algorithm
 *
 */

#include <acpi/acpi_bus.h>
#include <acpi/acpi_drivers.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/list.h>
#include <linux/timer.h>

#define ACPI_THERMAL_COMPONENT		0x04000000
#define ACPI_THERMAL_CLASS		"thermal_zone"
#define ACPI_THERMAL_DRIVER_NAME	"ACPI Thermal Zone Driver"
#define ACPI_THERMAL_DEVICE_NAME	"Thermal Zone"
#define ACPI_THERMAL_FILE_STATE		"state"
#define ACPI_THERMAL_FILE_TEMPERATURE	"temperature"
#define ACPI_THERMAL_FILE_TRIP_POINTS	"trip_points"
#define ACPI_THERMAL_FILE_COOLING_MODE	"cooling_mode"
#define ACPI_THERMAL_FILE_POLLING_FREQ	"polling_frequency"
#define ACPI_THERMAL_NOTIFY_TEMPERATURE	0x80
#define ACPI_THERMAL_NOTIFY_THRESHOLDS	0x81
#define ACPI_THERMAL_NOTIFY_DEVICES	0x82
#define ACPI_THERMAL_NOTIFY_CRITICAL	0xF0
#define ACPI_THERMAL_NOTIFY_HOT		0xF1
#define ACPI_THERMAL_MODE_ACTIVE	0x00
#define ACPI_THERMAL_MODE_PASSIVE	0x01
#define ACPI_THERMAL_MODE_CRITICAL   	0xff
#define ACPI_THERMAL_PATH_POWEROFF	"/sbin/poweroff"

#define ACPI_MAX_CHAR 5
#define ACPI_THERMAL_MAX_ACTIVE	10
#define ACPI_THERMAL_MAX_LIMIT_STR_LEN 65

#define KELVIN_TO_CELSIUS(t) \
	(long)(((long)t-2732 >= 0) ? ((long)t-2732+5)/10 \
	: ((long)t-2732-5)/10)
#define CELSIUS_TO_KELVIN(t)	((t+273)*10)

struct acpi_thermal_state {
	u8 critical:1;
	u8 hot:1;
	u8 passive:1;
	u8 active:1;
	u8 reserved:4;
	int active_index;
};

struct acpi_thermal_state_flags {
	u8 valid:1;
	u8 enabled:1;
	u8 reserved:6;
};

struct acpi_thermal_critical {
	struct acpi_thermal_state_flags flags;
	unsigned long temperature;
};

struct acpi_thermal_hot {
	struct acpi_thermal_state_flags flags;
	unsigned long temperature;
};

struct acpi_thermal_passive {
	struct acpi_thermal_state_flags flags;
	unsigned long temperature;
	unsigned long tc1;
	unsigned long tc2;
	unsigned long tsp;
	struct acpi_handle_list devices;
};

struct acpi_thermal_active {
	struct acpi_thermal_state_flags flags;
	unsigned long temperature;
	struct acpi_handle_list devices;
};

struct acpi_thermal_trips {
	struct acpi_thermal_critical critical;
	struct acpi_thermal_hot hot;
	struct acpi_thermal_passive passive;
	struct acpi_thermal_active active[ACPI_THERMAL_MAX_ACTIVE];
};

struct acpi_thermal_flags {
	u8 cooling_mode:1;	/* _SCP */
	u8 devices:1;		/* _TZD */
	u8 reserved:6;
};

/*By default kernel increments CPU throttlinby one step at a time*/
#define THROTTLE_INCREMENT_STEP 1
#define THROTTLE_DECREMENT_STEP -1

/*Support for absolute or relative throttlestates
  Relative is the one used by default kernel algorithm*/
#define THERMAL_MAX_THROTTLE_TYPE 2
enum throttle_set_type { THROTTLE_ABS, THROTTLE_REL };

/*Permissible Step range for relative throttling*/
#define MAX_THROTTLE_STEP 9
#define MIN_THROTTLE_STEP -9

/* --------------------------------------------------------------------------
			Component Driver Interface
   -------------------------------------------------------------------------- */
/*Reserved */
#define DEVICE_CAPS_RESERVED              0x000000
/*Has H/W thermal trip/throttle */
#define DEVICE_CAPS_HW_THROTTLE           0x000001
/*Has S/W programmable thermal events */
#define DEVICE_CAPS_SW_PGM_THERMAL_EVENTS 0x000002
/*Has thermal throttle controls (Non/Linear) */
#define DEVICE_CAPS_THROTTLE_CONTROLS     0x000004
/*Active device (e.g. Fan, Pump)" */
#define DEVICE_CAPS_ACTIVE                0x000008
/*Has monotonic throttle states */
#define DEVICE_CAPS_MONOTONIC_STATES      0x000010

typedef int (*thermal_device_set_throttle_limit) (enum throttle_set_type type,
						  void *data, const char *buf);
typedef int (*thermal_device_get_throttle_limit) (void *data, char *buf);
typedef int (*thermal_device_get_max_throttle_limit) (void *data, char *buf);
typedef int (*thermal_device_get_caps) (void *data, char *buf);

struct thermal_device_ops {
	/*Set new throttle limit */
	thermal_device_set_throttle_limit setthrottlelimit;
	/*Get current throttle limit */
	thermal_device_get_throttle_limit getthrottlelimit;
	/*Get maximum suppoted throttle state */
	thermal_device_get_max_throttle_limit getmaxthrottlelimit;
	/*Get device capabilities */
	thermal_device_get_caps getcaps;
};

struct thermal_device_driver {
	struct list_head node;	/*Linked list handling */
	unsigned long class;	/*Device class */
	struct thermal_device_ops *ops;	/*Callback functions */
};

struct thermal_device_info {
	char name[ACPI_MAX_CHAR];	  /*ACPI name */
	void *private;		          /*Device specific data */
	struct thermal_device_driver *dd; /*Device driver info */
	/*Info common to same devices in different TZs */
	struct thermal_device *common;
	struct kobject kobj;	          /*Sysfs stuff */
};

struct thermal_device {
	struct list_head node;	        /*Linked list handling */
	enum throttle_set_type type;	/*Absoulute/Relative */
	struct thermal_device_info info;/*name + driver details + kobj */
};

enum participant_class { THERMAL_UNKNOWN_CLASS, THERMAL_SKIN_CLASS,
	THERMAL_SYSTEM_CLASS, THERMAL_CPU_CLASS, THERMAL_MEM_CLASS,
	THERMAL_LCD_CLASS, THERMAL_GFX_CLASS, THERMAL_WLAN_CLASS,
	THERMAL_WWAN_CLASS, THERMAL_ACTIVE_CLASS
};

enum action_type { THERMAL_ACTION_ADD, THERMAL_ACTION_RMV };

/*Interface functions for component registration*/

/*
 * thermal_register_device
 * ----------------------
 * Interface function for devices like CPU,LCD ... to register
 * name : ACPI name like CPU0
 * class : class as defined in thermal.h
 * private: context of the device
 * return : a pointer to new thermal device
 */
struct thermal_device *thermal_register_device(const char *name,
					       enum participant_class class,
					       void *private);
/*
 * thermal_unregister_device
 * ----------------------
 * Interface function for devices like CPU,LCD ... to unregister
 * name : ACPI thermal device *
 */
int thermal_unregister_device(struct thermal_device *td);

/*
 * thermal_register_device_driver
 * -----------------------------
 * Interface function for platform sensor driver to unregister
 * dd : thermal_device_driver instance as defined in thermal.h
 */
int thermal_register_device_driver(struct thermal_device_driver *dd);

/*
 * thermal_unregister_device_driver
 * -----------------------------
 * Interface function for platform sensor driver to unregister
 * dd : thermal_device_driver instance as defined in thermal.h
 */
int thermal_unregister_device_driver(struct thermal_device_driver *dd);

/* --------------------------------------------------------------------------
				Sensor Driver Interface
   -------------------------------------------------------------------------- */

/*Sensor capabilities*/

/* Reserved */
#define SENSOR_CAPS_RESERVED 0x000000
/*Can support multiple participant devices */
#define SENSOR_CAPS_MULTIPLE_DEVICES 0x000001
/*Can report temperature */
#define SENSOR_CAPS_REPORT_TEMP 0x000002
/*Reports relative or absolute temperature */
#define SENSOR_CAPS_REPORT_ABS_TEMP 0x000004
/*Reports average or instantaneous temp */
#define SENSOR_CAPS_REPORT_INST_TEMP 0x000008
/* Programmable sample interval */
#define SENSOR_CAPS_PGM_SAMPLE_INT 0x000010
/*Can report rate-of-change */
#define SENSOR_CAPS_REPORT_RATE 0x000040
/* Has H/W thermal trip/throttle */
#define SENSOR_CAPS_HW_THERMAL_TRIP 0x010000
/*Has S/W programmable thermal events */
#define SENSOR_CAPS_SW_PGM_THERMAL_EVENTS 0x020000
/*Has ACPI thermal trip points */
#define SENSOR_CAPS_ACPI_THERMAL_TRIPS 0x040000

/*Sensor temperature base*/
#define SENSOR_TEMPBASE_ABS_KELVIN  0x00
#define SENSOR_TEMPBASE_REL_KELVIN  0x01
#define SENSOR_TEMPBASE_ABS_CELSIUS 0x02
#define SENSOR_TEMPBASE_REL_CELSIUS 0x03

enum aux_type { THERMAL_AUX0, THERMAL_AUX1 };

typedef int (*thermal_sensor_set_aux_trip) (const char *name, const int auxtype,
					    const char *buf);
typedef int (*thermal_sensor_get_aux_trip) (const char *name, const int auxtype,
					    char *buf);
typedef int (*thermal_sensor_get_temp_base) (const char *name, char *buf);
typedef int (*thermal_sensor_get_caps) (const char *name, char *buf);

struct thermal_sensor_ops {
	thermal_sensor_set_aux_trip setauxtrip;	 /*Get current AUX trip */
	thermal_sensor_get_aux_trip getauxtrip;	 /*Set new AUX trip */
	thermal_sensor_get_temp_base gettempbase;/*Get temperature base (K/c) */
	thermal_sensor_get_caps getcaps;	 /*Get sensor capabilities */
};
/*Interface functions for platform sensor driver registration*/

/*
 * thermal_register_sensor
 * ----------------------
 * Interface function for platform sensor driver to register
 * class: as defined in thermal.h
 * name : ACPI Thermal zone name like TZ00
 * ops: callback function as defined in platform sensor driver
 */
int thermal_register_sensor(enum participant_class class, const char *name,
			    struct thermal_sensor_ops *ops);

/*
 * thermal_unregister_sensor
 * ----------------------
 * Interface function for platform sensor driver to unregister
 * name : ACPI Thermal zone name like TZ00
 */
int thermal_unregister_sensor(const char *name);

/* --------------------------------------------------------------------------
			Configuration Interface
   -------------------------------------------------------------------------- */

#define THERMAL_SYSFS_CONFIG "config"

#define THERMAL_MAX_MODE 2
enum thermal_userenabled { THERMAL_USER_DISABLED, THERMAL_USER_ENABLED };
enum thermal_mode { THERMAL_USERSPACE, THERMAL_KERNELSPACE };

struct thermal_config {
	u8 userenabled:1;   /* 1 if user application is allowed to take over */
	u8 mode:1;	    /* 0 if userspace algorithm has taken over */
	struct kobject kobj;/*Syfs stuff */
};

/*
 * thermal_set_userenabled
 * -----------------------
 * Interface function for platform sensor driver to disble userspace algo
 * ue: enable / disable userspace algo based on BIOS configuration
 */
int thermal_set_userenabled(enum thermal_userenabled um);

/* --------------------------------------------------------------------------
			Thermal zone definition
   -------------------------------------------------------------------------- */
struct acpi_thermal {
	struct list_head node;	        /*Linkedlist management */
	struct acpi_device *device;	/*ACPI device info */
	acpi_bus_id name;	        /*ACPI device name (eg:- TZ00) */
	unsigned long temperature;	/*Current temperature */
	unsigned long last_temperature;	/*Previous temperature */
	unsigned long polling_frequency;/*Polling fequency */
	u8 cooling_mode;	        /*Active or passive */
	volatile u8 zombie;	       /*Flag for avoiding new deferred task */
	struct acpi_thermal_flags flags;/*See definition above */
	struct acpi_thermal_state state;/*See definition above */
	struct acpi_thermal_trips trips;/*_CRT,_HOT,_PSV*/
	struct acpi_handle_list devices;/* _TZD */
	struct timer_list timer;	/*Timer for polling thermal zone */
	struct thermal_device_info **info;/*Array of devices in TZ */
	/*Call back functions to platform sensor driver */
	struct thermal_sensor_ops *ops;
	enum participant_class class;/*Participant (device + sensor) class */
	struct kobject kobj;	        /*Syfs stuff */
};
